#include <windows.h>
#include <commdlg.h>
#include <shellapi.h>
#include <stdlib.h>
#include "dragdrop.h"

#define APPLICATION_NAME "Dimke Drag and Drop"
#define WINDOW_CX (GetSystemMetrics(SM_CXSCREEN)/2)
#define WINDOW_CY (GetSystemMetrics(SM_CYSCREEN)*3/4)
#define WINDOW_X (GetSystemMetrics(SM_CXSCREEN) - WINDOW_CX - 10) // Rechtsbndig mit Rand
#define WINDOW_Y 10

#define EDITID   1
#define UNTITLED "(untitled)"
#define MAX_TEXT_LENGTH 32000

long FAR PASCAL _export WndProc      (HWND, UINT, UINT, LONG);
BOOL FAR PASCAL _export AboutDlgProc (HWND, UINT, UINT, LONG);
void							WMDropFiles( WPARAM WParam, HWND hWnd);
void 							DragDropFileInitialize (HWND);
BOOL 							DragDropFileOpenDlg    (HWND, LPSTR, LPSTR);
BOOL DragDropFileSaveDlg    (HWND, LPSTR, LPSTR);
BOOL DragDropFileRead       (HWND, LPSTR);
BOOL DragDropFileWrite      (HWND, LPSTR);

static char szAppName [] = "Dimke Drag And Drop";
static HWND hDlgModeless;
static OPENFILENAME ofn;

int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance,
						  LPSTR lpszCmdLine, int nCmdShow) {
	  MSG      msg;
	  HWND     hwnd;
	  HANDLE   hAccel;
	  WNDCLASS wndclass;

	  if (!hPrevInstance) {
			 wndclass.style         = CS_HREDRAW | CS_VREDRAW;
			 wndclass.lpfnWndProc   = WndProc;
			 wndclass.cbClsExtra    = 0;
			 wndclass.cbWndExtra    = 0;
			 wndclass.hInstance     = hInstance;
			 wndclass.hIcon         = LoadIcon (hInstance, "Notepad");
			 wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
			 wndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
			 wndclass.lpszMenuName  = "MainMenu";
			 wndclass.lpszClassName = szAppName;

			 RegisterClass (&wndclass);
	  }

	  hwnd = CreateWindow (szAppName, NULL,
								  WS_OVERLAPPEDWINDOW,
								  WINDOW_X,      // Fensterposition x
								  WINDOW_Y,      // Fensterposition y
								  WINDOW_CX,     //FensterpositionBreite
								  WINDOW_CY,     //FensterpositionHoehe
								  NULL, NULL, hInstance, lpszCmdLine);

	  ShowWindow (hwnd, nCmdShow);
	  UpdateWindow (hwnd);

	  hAccel = LoadAccelerators (hInstance, "Tastenkuerzel");

	  while (GetMessage (&msg, NULL, 0, 0)) {
			 if (hDlgModeless == NULL || !IsDialogMessage (hDlgModeless, &msg)) {
					if (!TranslateAccelerator (hwnd, hAccel, &msg)) {
						  TranslateMessage (&msg);
						  DispatchMessage (&msg);
					}
			 }
	  }
	  return msg.wParam;
}

void DoCaption (HWND hwnd, char *szTitleName) {
	  char szCaption [64 + _MAX_FNAME + _MAX_EXT];

	  wsprintf (szCaption, "%s - %s", (LPSTR) szAppName,
					(LPSTR) (szTitleName [0] ? szTitleName : UNTITLED));

	  SetWindowText (hwnd, szCaption);
}

void OkMessage (HWND hwnd, char *szMessage, char *szTitleName) {
	  char szBuffer [64 + _MAX_FNAME + _MAX_EXT];

	  wsprintf (szBuffer, szMessage,
					(LPSTR) (szTitleName [0] ? szTitleName : UNTITLED));

	  MessageBox (hwnd, szBuffer, szAppName, MB_OK | MB_ICONEXCLAMATION);
}

short AskAboutSave (HWND hwnd, char *szTitleName) {
	  char  szBuffer [64 + _MAX_FNAME + _MAX_EXT];
	  short nReturn;

	  wsprintf (szBuffer, "Save current changes in %s?",
					(LPSTR) (szTitleName [0] ? szTitleName : UNTITLED));

	  nReturn = MessageBox (hwnd, szBuffer, szAppName,
									MB_YESNOCANCEL | MB_ICONQUESTION);

	  if (nReturn == IDYES)
			 if (!SendMessage (hwnd, WM_COMMAND, IDM_SAVE, 0L))
					nReturn = IDCANCEL;

	  return nReturn;
}

long FAR PASCAL _export WndProc (HWND hwnd, UINT message, UINT wParam, LONG lParam) {
	  static BOOL    bNeedSave = FALSE;
	  static char    szFileName  [_MAX_PATH];
	  static char    szTitleName [_MAX_FNAME + _MAX_EXT];
	  static FARPROC lpfnAboutDlgProc;
	  static HANDLE  hInst;
	  static HWND    hwndEdit;
	  static int     iOffset;
	  static UINT    messageFindReplace;
	  LONG           lSelect;
	  LPFINDREPLACE  lpfr;
	  WORD           wEnable;

	  switch (message) {
			 case WM_CREATE:
								 // Get About dialog instance address

					hInst = ((LPCREATESTRUCT) lParam)->hInstance;
					lpfnAboutDlgProc = MakeProcInstance ((FARPROC) AboutDlgProc, hInst);

								 // Create the edit control child window

					DragAcceptFiles(hwnd, TRUE);

					hwndEdit = CreateWindow ("edit", NULL,
								 WS_CHILD | WS_VISIBLE | WS_HSCROLL | WS_VSCROLL |
										WS_BORDER | ES_LEFT | ES_MULTILINE |
										ES_NOHIDESEL | ES_AUTOHSCROLL | ES_AUTOVSCROLL,
								 0, 0, 0, 0,
								 hwnd, EDITID, hInst, NULL);

					SendMessage (hwndEdit, EM_LIMITTEXT, MAX_TEXT_LENGTH, 0L);

								 // Initialize common dialog box stuff

					DragDropFileInitialize (hwnd);

					messageFindReplace = RegisterWindowMessage (FINDMSGSTRING);

								 // Process command line

					lstrcpy (szFileName, (LPSTR)
								(((LPCREATESTRUCT) lParam)->lpCreateParams));

					if (lstrlen (szFileName) > 0) {
						  GetFileTitle (szFileName, szTitleName, sizeof (szTitleName));

						  if (!DragDropFileRead (hwndEdit, szFileName))
								 OkMessage (hwnd, "File %s cannot be read!", szTitleName);
					}

					DoCaption (hwnd, szTitleName);
					return 0;

			 case WM_SETFOCUS:
					SetFocus (hwndEdit);
					return 0;

			 case WM_SIZE:
					MoveWindow (hwndEdit, 0, 0, LOWORD (lParam),
														 HIWORD (lParam), TRUE);
					return 0;

			 case WM_INITMENUPOPUP:
					if(lParam==1) {
										// Enable Undo if edit control can do it

								 EnableMenuItem (wParam, IDM_UNDO,
										SendMessage (hwndEdit, EM_CANUNDO, 0, 0L) ?
											  MF_ENABLED : MF_GRAYED);

										// Enable Paste if text is in the clipboard

								 EnableMenuItem (wParam, IDM_PASTE,
										IsClipboardFormatAvailable (CF_TEXT) ?
											  MF_ENABLED : MF_GRAYED);

										// Enable Cut, Copy, and Del if text is selected

								 lSelect = SendMessage (hwndEdit, EM_GETSEL, 0, 0L);
								 wEnable = HIWORD (lSelect) != LOWORD (lSelect) ?
													 MF_ENABLED : MF_GRAYED;

								 EnableMenuItem (wParam, IDM_CUT,  wEnable);
								 EnableMenuItem (wParam, IDM_COPY, wEnable);
								 EnableMenuItem (wParam, IDM_DEL,  wEnable);

										// Enable Select All if edit control is not empty

								 EnableMenuItem (wParam, IDM_SELALL,
										GetWindowTextLength (hwndEdit) >0 ?
											  MF_ENABLED : MF_GRAYED);
					}
					return 0;

			 case WM_DROPFILES:
				WMDropFiles(wParam, hwndEdit);
				break;

			 case WM_COMMAND :
										// Messages from edit control

					if (LOWORD (lParam) && wParam == EDITID) {
						  switch (HIWORD (lParam)) {
								 case EN_UPDATE:
										bNeedSave = TRUE;
										return 0;

								 case EN_ERRSPACE:
								 case EN_MAXTEXT:
										MessageBox (hwnd, "Edit control out of space.",
													 szAppName, MB_OK | MB_ICONSTOP);
										return 0;
						  }
						  break;
					}

					switch (wParam) {
										// Messages from File menu

						  case IDM_NEW:
								 if (bNeedSave && IDCANCEL ==
											  AskAboutSave (hwnd, szTitleName))
										return 0;

								 SetWindowText (hwndEdit, "\0");
								 szFileName [0]  = '\0';
								 szTitleName [0] = '\0';
								 DoCaption (hwnd, szTitleName);
								 bNeedSave = FALSE;
								 return 0;

						  case IDM_OPEN:
								 if (bNeedSave && IDCANCEL ==
											  AskAboutSave (hwnd, szTitleName))
										return 0;

								 if (DragDropFileOpenDlg (hwnd, szFileName, szTitleName)) {
										if (!DragDropFileRead (hwndEdit, szFileName)) {
											  OkMessage (hwnd, "Could not read file %s!",
																	 szTitleName);
											  szFileName  [0] = '\0';
											  szTitleName [0] = '\0';
										}
								 }

								 DoCaption (hwnd, szTitleName);
								 bNeedSave = FALSE;
								 return 0;

						  case IDM_SAVE:
								 if (szFileName [0]) {
										if (DragDropFileWrite (hwndEdit, szFileName)) {
											  bNeedSave = FALSE;
											  return 1;
										}
										else
											  OkMessage (hwnd, "Could not write file %s",
																	 szTitleName);
										return 0;
								 }
																  // fall through
						  case IDM_SAVEAS:
								 if (DragDropFileSaveDlg (hwnd, szFileName, szTitleName)) {
										DoCaption (hwnd, szTitleName);

										if (DragDropFileWrite (hwndEdit, szFileName)) {
											  bNeedSave = FALSE;
											  return 1;
										}
										else
											  OkMessage (hwnd, "Could not write file %s",
																	 szTitleName);
								 }
								 return 0;

						  case IDM_EXIT:
								 SendMessage (hwnd, WM_CLOSE, 0, 0L);
								 return 0;

										// Messages from Edit menu

						  case IDM_UNDO:
								 SendMessage (hwndEdit, WM_UNDO, 0, 0L);
								 return 0;

						  case IDM_CUT:
								 SendMessage (hwndEdit, WM_CUT, 0, 0L);
								 return 0;

						  case IDM_COPY:
								 SendMessage (hwndEdit, WM_COPY, 0, 0L);
								 return 0;

						  case IDM_PASTE:
								 SendMessage (hwndEdit, WM_PASTE, 0, 0L);
								 return 0;

						  case IDM_DEL:
								 SendMessage (hwndEdit, WM_CLEAR, 0, 0L);
								 return 0;

						  case IDM_SELALL:
								 SendMessage (hwndEdit, EM_SETSEL, 0,
													 MAKELONG (0, 32767));
								 return 0;

										// Messages from Help menu

						  case IDM_HELP:

						  case IDM_ABOUT:
								 DialogBox (hInst, "AboutBox", hwnd, lpfnAboutDlgProc);
								 return 0;
					}
					break;

			 case WM_CLOSE:
					if (!bNeedSave || IDCANCEL != AskAboutSave (hwnd, szTitleName))
						  DestroyWindow (hwnd);

					return 0;

			 case WM_QUERYENDSESSION:
					if (!bNeedSave || IDCANCEL != AskAboutSave (hwnd, szTitleName))
						  return 1L;

					return 0;

			 case WM_DESTROY:
					PostQuitMessage (0);
					return 0;

			 default:
								 // Process "Find-Replace" messages
					break;
	  }
	  return DefWindowProc (hwnd, message, wParam, lParam);
}

BOOL FAR PASCAL _export AboutDlgProc (HWND hDlg, UINT message, UINT wParam, LONG lParam) {
	  switch (message) {
			 case WM_INITDIALOG:
					return TRUE;

			 case WM_COMMAND:
					switch (wParam) {
						  case IDCANCEL:
						  case IDOK:
								 EndDialog (hDlg, 0);
								 return TRUE;
					}
					break;
	  }
	  return FALSE;
}


void DragDropFileInitialize (HWND hwnd) {
	  static char *szFilter[] = { "Text Files (*.txt)",  "*.txt",
											"Config Files (*.cfg)", "*.cfg",
											"MP3 File Lists (*.m3u)", "*.m3u",
											"All Files (*.*)",     "*.*",
											"" };

	  ofn.lStructSize       = sizeof (OPENFILENAME);
	  ofn.hwndOwner         = hwnd;
	  ofn.hInstance         = NULL;
	  ofn.lpstrFilter       = szFilter [0];
	  ofn.lpstrCustomFilter = NULL;
	  ofn.nMaxCustFilter    = 0;
	  ofn.nFilterIndex      = 0;
	  ofn.lpstrFile         = NULL;          // Set in Open and Close functions
	  ofn.nMaxFile          = _MAX_PATH;
	  ofn.lpstrFileTitle    = NULL;          // Set in Open and Close functions
	  ofn.nMaxFileTitle     = _MAX_FNAME + _MAX_EXT;
	  ofn.lpstrInitialDir   = NULL;
	  ofn.lpstrTitle        = NULL;
	  ofn.Flags             = 0;             // Set in Open and Close functions
	  ofn.nFileOffset       = 0;
	  ofn.nFileExtension    = 0;
	  ofn.lpstrDefExt       = "txt";
	  ofn.lCustData         = 0L;
	  ofn.lpfnHook          = NULL;
	  ofn.lpTemplateName    = NULL;
}

BOOL DragDropFileOpenDlg (HWND hwnd, LPSTR lpstrFileName, LPSTR lpstrTitleName) {
	  ofn.hwndOwner         = hwnd;
	  ofn.lpstrFile         = lpstrFileName;
	  ofn.lpstrFileTitle    = lpstrTitleName;
	  ofn.Flags             = OFN_CREATEPROMPT;

	  return GetOpenFileName (&ofn);
}

BOOL DragDropFileSaveDlg (HWND hwnd, LPSTR lpstrFileName, LPSTR lpstrTitleName) {
	  ofn.hwndOwner         = hwnd;
	  ofn.lpstrFile         = lpstrFileName;
	  ofn.lpstrFileTitle    = lpstrTitleName;
	  ofn.Flags             = OFN_OVERWRITEPROMPT;

	  return GetSaveFileName (&ofn);
}

static long DragDropFileLength (int hFile) {
	  long lCurrentPos = _llseek (hFile, 0L, 1);
	  long lFileLength = _llseek (hFile, 0L, 2);

	  _llseek (hFile, lCurrentPos, 0);

	  return lFileLength;
}

BOOL DragDropFileRead (HWND hwndEdit, LPSTR lpstrFileName) {
	  long   lLength;
	  HANDLE hBuffer;
	  int    hFile;
	  LPSTR  lpstrBuffer;

	  if (-1 == (hFile = _lopen (lpstrFileName, OF_READ | OF_SHARE_DENY_WRITE)))
			 return FALSE;

	  if ((lLength = DragDropFileLength (hFile)) >= MAX_TEXT_LENGTH) {
			 _lclose (hFile);
			 return FALSE;
	  }

	  if (NULL == (hBuffer = GlobalAlloc (GHND, lLength + 1))) {
			 _lclose (hFile);
			 return FALSE;
	  }

	  lpstrBuffer = GlobalLock (hBuffer);
	  _lread (hFile, lpstrBuffer, (WORD) lLength);
	  _lclose (hFile);
	  lpstrBuffer [(WORD) lLength] = '\0';

	  SetWindowText (hwndEdit, lpstrBuffer);
	  GlobalUnlock (hBuffer);
	  GlobalFree (hBuffer);

	  return TRUE;
}

BOOL DragDropFileWrite (HWND hwndEdit, LPSTR lpstrFileName) {
	  HANDLE hBuffer;
	  int    hFile;
	  LPSTR  lpstrBuffer;
	  WORD   wLength;

	  if (-1 == (hFile = _lcreat (lpstrFileName, 0)))
			 return FALSE;

	  wLength = GetWindowTextLength (hwndEdit);
	  hBuffer = (HANDLE) SendMessage (hwndEdit, EM_GETHANDLE, 0, 0L);
	  lpstrBuffer = (LPSTR) LocalLock (hBuffer);

	  if (wLength != _lwrite (hFile, lpstrBuffer, wLength)) {
			 _lclose (hFile);
			 return FALSE;
	  }

	  _lclose (hFile);
	  LocalUnlock (hBuffer);

	  return TRUE;
}

void WMDropFiles( WPARAM WParam, HWND hwndEdit) {
	  struct FileList *FLptr, *FLCurrent=NULL, *FLHead=NULL;
	  struct List *liFiles;
	  int TotalNumberOfFiles;
	  int nFileLength;
	  char *pszFileName;
	  POINT pPoint;
	  BOOL inClientArea;
	  int i, mid_string;
	  HANDLE Handle = (HANDLE)WParam;
				//WParam contains the Handle to the
				//internal data structure which has
				//information about the dropped file(s)
	  TotalNumberOfFiles = DragQueryFile( Handle , -1, NULL, 0 );

	  for ( i = 0; i < TotalNumberOfFiles; i++ ) {
			nFileLength  = DragQueryFile( Handle , i , NULL, 0 );
			pszFileName = (char*)malloc(nFileLength + 1);
			DragQueryFile( Handle , i , pszFileName , nFileLength+1 );

			for ( mid_string = 0; mid_string < nFileLength; mid_string++ ) {
				SendMessage(hwndEdit, WM_CHAR, pszFileName[mid_string], 0);
			}

			SendMessage(hwndEdit, WM_CHAR, '\n', 0);
	  }
	  DragFinish(Handle);
}
